local addonName, addon = ...
local _G = _G

----------------------------------------------------------------------------------
-- EVENTS
----------------------------------------------------------------------------------

function addon.OnLoad(self)
	-- self:RegisterForDrag("LeftButton", "RightButton");
	self:RegisterForClicks("AnyUp");
	self.forceUncheck = false;
	self.IsUsable = true;
	self.flashing = 0;
	self.nextUpdate = 0;
	self.cursorX = -1;
	self.cursorY = -1;
	self.HasNormalCursor = true;
	self.HasShiftCursor = false;
	self.HasControlCursor = false;
	MagnetButtonProperties:SetFrameStrata("TOOLTIP");
end

-- This function adjusts x and y during the drag and determines if near other buttons
function addon.DragUpdateLocation(self, distanceResolution)
	local xpos, ypos = GetCursorPosition();
	local rectThis = Rectangle.CreateFromFrame(self);
	local scale = UIParent:GetEffectiveScale();
	local buttonScale = self:GetParent():GetScale();
	local count = 0;
	local found = false;
	
	-- scale
	xpos = xpos / scale;
	ypos = ypos / scale;

	-- adjust for offset point
	xpos = xpos - (self:GetWidth() / 2);
	ypos = ypos - (self:GetHeight() / 2);

	-- scan for other magnet button frames
	for index = 1, #self.OtherFrames do
		-- get the Rectangle of the other frame
		local rectFrame = self.OtherFrames[index]["rect"];
		local frameName = self.OtherFrames[index]["name"];	-- use or remove
		local targetFrame = getglobal(frameName);			-- use or remove
		local targetScale = targetFrame:GetScale();

		if (targetScale == buttonScale) then
			-- bIntersect set to true only means the other return values are not null.
			--local bIntersect, offsetX, offsetY = Rectangle.Intersect(rectThis, rectFrame);
			local bIntersect, offsetX, offsetY = Rectangle.ZoneIntersect(rectThis, rectFrame, distanceResolution);
			local oldX = xpos;
			local oldY = ypos;
			-- We're intersection with another button
			if (bIntersect) then
				--local side = "";
				count = count + 1;
			
				-- 'a' squared plus 'b' squared is equal to 'c' squared.
				local length = math.sqrt(offsetX * offsetX + offsetY * offsetY);

				-- What side?
				local angle = math.atan2(offsetX, offsetY);
				angle = 180 - (angle * (180 / 3.141592653589793));
			
				if (angle > 45) and (angle <= 135) then
					--side = "left";
					xpos = rectFrame["x"] - rectFrame["width"] ;
					ypos = rectFrame["y"];
				elseif (angle > 135) and (angle <= 225) then
					--side = "bottom";
					ypos = rectFrame["y"] - rectFrame["width"];
					xpos = rectFrame["x"];
				elseif (angle > 225) and (angle <= 315) then
					--side = "right";
					xpos = rectFrame["x"] + rectFrame["width"];
					ypos = rectFrame["y"];
				else
					--side = "top";
					ypos = rectFrame["y"] + rectFrame["width"];
					xpos = rectFrame["x"];
				end
				ypos = ypos * buttonScale;
				xpos = xpos * buttonScale;
				found = true;
			end

			-- Prevent two buttons from having the same location
			if (bIntersect) then
				local i;
				local targetRect = Rectangle.Create(xpos, ypos, 0, 0);
				for i = 1, #self.OtherFrames do
					local rectTest = self.OtherFrames[i]["rect"];
					if (Rectangle.IsSameLocation(targetRect, rectTest)) then
						xpos = oldX;
						ypos = oldY;
						found = false;
						break;
					end
				end
			end
		end
	end
	return xpos, ypos, found;
end

function addon.OnReceiveDrag(self)
	-- Cursor is holding a spell
	local infoType, info1, info2, info3 = GetCursorInfo();

	-- infoType = "flyout"
	-- flyout, 9, 132161 (Call Pet)... What's the 9 mean, the 132161 is probably a iconID
	DoIt.Debug("OnReceiveDrag: "..tostring(infoType)..", "..tostring(info1)..", "..tostring(info2)..", "..tostring(info3)..", "..tostring(inLockdown));
	
	-- Cancel if in combat lockdown
	if (InCombatLockdown()) then
		ClearCursor();
		return;
	end

	-- Setup Button
	if (infoType == "mount" and tonumber(info1) == 268435455) then
		addon.SetMacroText(self:GetParent(), "/script SummonRandomFavoriteMount()", "413588", "Summon Random Favorite Mount", nil, "macro", "/dismount");
		addon.SaveButton(self);
	elseif (infoType == "spell") then
		local spellName = GetSpellBookItemName(info1, BOOKTYPE_SPELL);
		if (info1 == 0) then
			spellName = GetSpellInfo(info3);
		end

		--addon.SetButton(self, {
		--	type = "spell"
		--  spell = spellName
		--})
		--DoIt.Debug(format("OnReceiveDrag: SetSpellBookItem: "..tostring(spellName)));
		
		addon.SetSpellBookItem(self:GetParent(), spellName);
		addon.SaveButton(self);
		addon.UpdateFlash(self);
	elseif (infoType == "item") then
		local hasItem = CursorHasItem()
		local itemID, toyName, textureId, unknown2;
		if (not DoIt.IsClassic) then
			itemID, toyName, textureId, unknown2 = C_ToyBox.GetToyInfo(info1);
		end
		if (hasItem) then
			addon.SetItem(self:GetParent(), info2);
		elseif (toyName) then
			-- FIX TO MAKE TOYS WORK IS TO TURN THEM INTO A "/USE {toyname}" macro
			addon.SetMacroText(self:GetParent(), "/use "..toyName, textureId);
			self:SetAttribute("tooltip", toyName);
		end
		addon.SaveButton(self);
	elseif (infoType == "mount") then
		local mountName, mountIndex, textureId = addon.FindMount(info1);
		--DoIt.Debug(format("Type_Mount: "..tostring(mountName)..", "..tostring(mountIndex)..", textureId = "..tostring(textureId)));
		if (mountName == "Chauffeured Mekgineer's Chopper") then
			addon.SetMacroText(self:GetParent(), "/use Summon Chauffeur", textureId, "Mount "..mountName, "", "macro", "/dismount");
		else
			addon.SetMacroText(self:GetParent(), "/use "..mountName, textureId, "Mount "..mountName, "", "macro", "/dismount");
		end
		addon.SaveButton(self);
	elseif (infoType == "macro") then
		local name, iconTexture, body, isLocal = GetMacroInfo(info1);
		--DoIt.Debug(format("Type_Macro: "..tostring(mountName)..", "..tostring(mountIndex)..", textureId = "..tostring(textureId)));
		addon.SetMacroName(self:GetParent(), name);
		-- Setup texture
		local icon = getglobal(self:GetName().."Icon");
		icon:SetTexture(iconTexture);
		self:SetAttribute("texture", icon:GetTexture());
		addon.SaveButton(self);
	elseif (infoType == "companion") then
		-- DoIt.Debug("OnReceiveDrag: SetCompanion");
		addon.SetCompanion(self:GetParent(), info2, info1);
		addon.SaveButton(self);
		addon.UpdateFlash(self);
	elseif (infoType == "flyout") then
		--DoIt.Debug("Flyout drop detected...")
		addon.SetFlyout(self:GetParent(), info1, info2);
		addon.SaveButton(self);
	elseif (infoType == "petaction") then
		if (addon.DraggingPetSpellIndex) then
			local spellId = info1;
			if (info2 == nil) then
				addon.SetPetAction(self:GetParent(), addon.DraggingPetSpellIndex, nil);
			else
				addon.SetPetAction(self:GetParent(), addon.DraggingPetSpellIndex, info1);
			end
		else
			addon.SetPetAction(self:GetParent(), info2, info1);
		end
		addon.SaveButton(self);
	end
	addon.UpdateState(self);
	addon.UpdateCooldown(self);

	ClearCursor();
end

function addon.GetOtherFrameLocations(self)
	local myTable = { };
	local trueIndex = 1;
	for index = 1, addon.MaxFrameIndex do
		local frameName = "MagnetButtonFrame"..tostring(index);
		local f = getglobal(frameName);
		if (f ~= nil) and (f:IsShown()) and (f ~= self:GetParent()) then
			myTable[trueIndex] = { };
			myTable[trueIndex]["name"] = frameName;
			myTable[trueIndex]["rect"] = Rectangle.CreateFromFrame(f);
			trueIndex = trueIndex + 1;
		end
	end
	return myTable;
end

function addon.IsButtonLocked(self)
	return false;
end

function addon.PreClick(self, button)

end

function addon.PostClick(self, button)

end

function addon.OnMouseDown(self, button)
	local shiftDown = IsShiftKeyDown();
	local ctrlDown  = IsControlKeyDown();
	local altDown   = IsAltKeyDown();
	--DoIt.Debug("OnMouseDown: "..tostring(button)..", "..tostring(self:GetScript("OnUpdate"))..", "..tostring(addon.OnUpdate));

	if (not addon.IsButtonLocked()) and (button == "LeftButton") and (shiftDown) then
		local parent = self:GetParent();
		local parentName = parent:GetName();
		self.IsMoving = true;
		self.OtherFrames = addon.GetOtherFrameLocations(self);
		--DoIt.Debug("MovingButton: "..tostring(parentName)..", "..tostring(#self.OtherFrames));
        -- self:LockHighlight();
 		self:RegisterForClicks();
 		self:Disable();
		parent:StartMoving();
	elseif (not self.IsMoving) and (button == "LeftButton") and ctrlDown and not (UnitAffectingCombat("player") or InCombatLockdown()) then
		addon.ShowProps(self);
	end
end

function addon.OnMouseUp(self)
	local parent = self:GetParent();
	local parentName = parent:GetName();

	if (self.IsMoving) then
		self.IsMoving = false;
		self.OtherFrames = nil;
		parent:StopMovingOrSizing();
		addon.SaveButton(self);
		self:Enable();
		self:RegisterForClicks("AnyUp");
        return;
	end
	
	-- Button handling for mounts
	if (self:GetAttribute("type") == "companion") then
		local spell = self:GetAttribute("spell");
		local value1 = self:GetAttribute("value1");
		local value2 = self:GetAttribute("value2");
		-- DoIt.Debug("Call companion: "..value1..", "..value2);
		if (DoIt.Player.hasAura(spell)) then
			DismissCompanion(value1, value2);
		else
			CallCompanion(value1, value2);
		end
	end	
end

function addon.OnEvent(self, event, arg1, arg2, ...)
	-- local spell, rank, displayName, icon, startTime, endTime, isTradeSkill, castID, interrupt = UnitCastingInfo("player");
	local tradeskillName, currentLevel, maxLevel;
	if (not DoIt.IsClassic) then
		tradeskillName, currentLevel, maxLevel = C_TradeSkillUI.GetTradeSkillLine()
	end
	
	local spell = addon.GetSpellBookItemName(self);
	local item = addon.GetItemName(self);
	local action = addon.GetPetAction(self);
	if (spell == nil) and (item == nil) and (action == nil) then
		addon.OnUpdate(self);
		return;
	end

	if (event == "SPELLS_CHANGED" or event == "SPELL_UPDATE_CHARGES") then
		addon.SetButtonItemCount(self);
		return;
	end
	
	if (event == "RUNE_POWER_UPDATE") and (arg1 ~= "player") then
		-- DoIt.Debug("UNIT_RUNIC_POWER, "..tostring(arg1)..", "..tostring(arg2));
		return;
	end
	if (event == "ACTIONBAR_UPDATE_STATE") then
		--DoIt.Debug("ACTIONBAR_UPDATE_STATE: "..tostring(arg1)..", "..tostring(arg2));
	end
	if (event == "ACTIONBAR_UPDATE_USABLE") then
		---DoIt.Debug("ACTIONBAR_UPDATE_USABLE: "..tostring(arg1)..", "..tostring(arg2));
	end
	if (event == "ACTIONBAR_UPDATE_COOLDOWN") then
		---DoIt.Debug("ACTIONBAR_UPDATE_COOLDOWN: "..arg1..", "..arg2);
	end
	if (event == "PLAYER_ENTER_COMBAT") then
		--if (IsAttackSpell(spell)) then
			--addon.StartFlash(self);
		--end
	elseif (event == "PLAYER_LEAVE_COMBAT") then
		--if (IsAttackSpell(spell)) then
			--addon.StopFlash(self);
		--end
	elseif (event == "START_AUTOREPEAT_SPELL") then
		if (IsAutoRepeatSpell(spell)) then
			--addon.StartFlash(self);
		end
	elseif (event == "STOP_AUTOREPEAT_SPELL") then
		if (addon.IsFlashing(self) and not IsAttackSpell(spell)) then
			--addon.StopFlash(self);
		end
	elseif (event == "CURRENT_SPELL_CAST_CHANGED") and (not SpellIsTargeting()) and (tradeskillName == "UNKNOWN") then
		self:SetChecked(false);
	elseif (event == "UI_ERROR_MESSAGE") then
		self:SetChecked(false);
	elseif (event == "UNIT_SPELLCAST_SUCCEEDED") then
		-- DoIt.Debug("UNIT_SPELLCAST_SUCCEEDED: "..tostring(arg2));
		if (arg1 == "player") and (self:GetAttribute("type") == "spell") and (self:GetAttribute("spell") == arg2) then
			-- DoIt.Debug("UNIT_SPELLCAST_SUCCEEDED: HOOKED");
		end
	elseif (event == "COMBAT_LOG_EVENT") and (arg2 == "SPELL_AURA_APPLIED") and (arg4 == UnitName("player")) and (arg10 == self:GetAttribute("spell") and (self:GetAttribute("type") == "spell"))  then
		-- DoIt.Debug("SPELL_AURA_APPLIED: "..tostring(arg10));
		self.AuraApplied = true;
		-- self:SetChecked(true);
	elseif (event == "COMBAT_LOG_EVENT") and (arg2 == "SPELL_AURA_REMOVED") and (arg4 == UnitName("player")) and (arg10 == self:GetAttribute("spell") and (self:GetAttribute("type") == "spell")) then
		-- DoIt.Debug("SPELL_AURA_REMOVED: "..tostring(arg10));
		self.AuraApplied = false;
		-- self:SetChecked(false);
	elseif (event == "COMPANION_UPDATE") and (arg1 ~= "MOUNT") then
		--local _, _, _, _, issummoned = GetCompanionInfo(self:GetAttribute("value1"), self:GetAttribute("value2"));
		-- DoIt.Debug("COMPANION_UPDATE: "..tostring(arg1)..", "..tostring(issummoned));
		self.AuraApplied = issummoned; -- Controls the check state
	end
	
	if (event == "BAG_UPDATE") and (item ~= nil) then
		addon.SetButtonItemCount(self);
	end
	
	addon.OnUpdate(self);
end

